function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { createHashHistory, createBrowserHistory, createMemoryHistory, getBaseHref, History } from './history';
import { innumerable, isHistory, once, getSessionStorage, setSessionStorage } from './util';
import { HistoryFix } from './types';
import { isFunction } from '.';
export let HistoryType;

(function (HistoryType) {
  HistoryType["hash"] = "hash";
  HistoryType["browser"] = "browser";
  HistoryType["memory"] = "memory";
})(HistoryType || (HistoryType = {}));

const RAINBOW_ROUTER_KEY = '__RAINBOW_ROUTER_GLOBAL__';

if (!window[RAINBOW_ROUTER_KEY]) {
  innumerable(window, RAINBOW_ROUTER_KEY, {
    historys: {}
  });
}

const RAINBOW_ROUTER_GLOBAL = window[RAINBOW_ROUTER_KEY];

function eachInterceptor(interceptors, location, callback, index, nexts) {
  let item = interceptors[index];
  if (!item) return callback(true);
  if (isFunction(item)) item = {
    interceptor: item,
    router: null
  };
  const cb = once(ok => {
    if (!ok) return callback(ok);
    if (isFunction(ok)) nexts.push(ok);
    eachInterceptor(interceptors, location, callback, index + 1, nexts);
  });
  if (item.router && item.router.isRunning) item.interceptor(location, cb);else cb(true);
}

function confirmInterceptors(interceptors, location, callback) {
  if (isHistory(interceptors)) interceptors = interceptors.interceptors;
  const nexts = [];
  interceptors = [...interceptors];
  return eachInterceptor(interceptors, location, ok => {
    nexts.forEach(next => next(ok));
    callback(ok);
  }, 0, nexts);
}

function createStackInfo(index, location) {
  return {
    pathname: location.pathname,
    search: location.search,
    index,
    timestamp: Date.now()
  };
}

function createHistory(options, fn, type) {
  const interceptors = [];
  const history = fn();
  history.isHistoryInstance = true;
  history.interceptors = interceptors;

  history.interceptorTransitionTo = function (interceptor, router) {
    const idx = this.interceptors.findIndex(v => v.interceptor === interceptor);

    if (idx < 0) {
      if (router && router.basename) {
        let parentRouter = null;
        this.interceptors.forEach(v => {
          if (v.router && router.basename.includes(v.router.basename)) {
            if (!parentRouter || parentRouter.basename < v.router.basename) parentRouter = v.router;
          }
        });
        if (router._updateParent) router._updateParent(parentRouter);
      }

      this.interceptors.push({
        interceptor,
        router
      });
    }

    return () => {
      const idx = this.interceptors.findIndex(v => v.interceptor === interceptor);

      if (~idx) {
        this.interceptors.splice(idx, 1);
        if (router.parent && router._updateParent) router._updateParent(null);
      }
    };
  };

  const needSession = type !== HistoryType.memory;
  const SessionStacksKey = `_RAINBOW_ROUTER_${type.toUpperCase()}_STACKS_`;
  history.stacks = needSession ? getSessionStorage(SessionStacksKey, true) || [] : [];
  const lastStackInfo = history.stacks[history.stacks.length - 1];

  if (!lastStackInfo || lastStackInfo.index !== history.index) {
    history.stacks.push(createStackInfo(history.index, history.location));
    if (needSession) setSessionStorage(SessionStacksKey, history.stacks);
  }

  innumerable(history, '_unlisten', history.listen(state => {
    if (!state) return;
    const {
      location,
      index
    } = state;
    const lastStackInfo = history.stacks[history.stacks.length - 1];

    if (lastStackInfo && index > lastStackInfo.index) {
      history.stacks.push(createStackInfo(index, location));
    } else {
      const idx = history.stacks.findIndex(v => v.index === index);

      if (~idx) {
        history.stacks.splice(idx, history.stacks.length, createStackInfo(index, location));
      }
    }

    if (needSession) setSessionStorage(SessionStacksKey, history.stacks); // console.log('[createHistory][listen]', location, action, index);
  }));
  innumerable(history, '_unblock', history.block(({
    action,
    location,
    callback
  }) => {
    location = _objectSpread({}, location);
    confirmInterceptors(interceptors, location, callback);
  }));
  return history;
}

function createHashHistoryNew(options) {
  if (RAINBOW_ROUTER_GLOBAL.historys.hash) {
    return RAINBOW_ROUTER_GLOBAL.historys.hash;
  }

  return RAINBOW_ROUTER_GLOBAL.historys.hash = createHistory(options, () => createHashHistory(options), HistoryType.hash);
}

function createBrowserHistoryNew(options) {
  if (RAINBOW_ROUTER_GLOBAL.historys.browser) {
    return RAINBOW_ROUTER_GLOBAL.historys.browser;
  }

  return RAINBOW_ROUTER_GLOBAL.historys.browser = createHistory(options, () => createBrowserHistory(options), HistoryType.browser);
}

function createMemoryHistoryNew(options) {
  return createHistory(options, () => createMemoryHistory(options), HistoryType.memory);
}

function getPossibleRouterMode() {
  if (RAINBOW_ROUTER_GLOBAL.historys.hash) return 'hash';
  if (RAINBOW_ROUTER_GLOBAL.historys.browser) return 'browser';
  return '';
}

export { createHashHistoryNew as createHashHistory, createBrowserHistoryNew as createBrowserHistory, createMemoryHistoryNew as createMemoryHistory, getBaseHref, getPossibleRouterMode, History, HistoryFix, confirmInterceptors }; // export {
//   createHashHistory,
//   createBrowserHistory,
//   createMemoryHistory,
//   History,
//   HistoryFix,
//   LocationState
// };