function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Actions represent the type of change to a location value.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#action
 */
export let Action;

(function (Action) {
  Action["Pop"] = "POP";
  Action["Push"] = "PUSH";
  Action["Replace"] = "REPLACE";
})(Action || (Action = {}));

// eslint-disable-next-line no-constant-condition
const readOnly = false
/** __DEV__* */
? obj => Object.freeze(obj) : obj => obj; // //////////////////////////////////////////////////////////////////////////////
// BROWSER
// //////////////////////////////////////////////////////////////////////////////

// const BeforeUnloadEventType = 'beforeunload';
const HashChangeEventType = 'hashchange';
const PopStateEventType = 'popstate';

/**
 * Browser history stores the location in regular URLs. This is the standard for
 * most web apps, but it requires some configuration on the server to ensure you
 * serve the same app at multiple URLs.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#createbrowserhistory
 */
export function createBrowserHistory(options = {}) {
  let {
    window = document.defaultView,
    hashType
  } = options;
  let globalHistory = window.history;

  function getIndexAndLocation() {
    let {
      pathname,
      search,
      hash
    } = window.location;
    let state = globalHistory.state || {};
    return [state.idx, readOnly({
      pathname,
      search,
      hash,
      state: state.usr || null,
      key: state.key || 'default'
    })];
  }

  let blockedPopTx = null;
  let blockedPopAp = null;
  let blockedPopDc = null;

  function handlePop(e) {
    let [nextIndex, nextLocation] = getIndexAndLocation();

    if (nextIndex == null) {
      if (index == null || isNaN(index)) index = 0;
      nextIndex = replaceHistoryState(nextLocation, index + 1);
    }

    let delta = index - nextIndex;
    let nextAction = delta > 0 ? Action.Pop : Action.Push;

    if (blockedPopDc) {
      blockedPopDc({
        index: nextIndex,
        location: nextLocation
      });
      blockedPopDc = null;
    } else if (blockedPopAp) {
      if (nextIndex === blockedPopAp.prevIndex) {
        go(blockedPopAp.delta);
        return;
      }

      blockedPopAp = null;
    } else if (blockers.length && delta) {
      const callback = ok => {
        blockedPopTx = null;

        if (ok) {
          applyTx(nextAction);
          return;
        }

        blockedPopAp = {
          action,
          index,
          location,
          prevIndex: nextIndex,
          delta
        };
        go(blockedPopAp.delta);
      };

      blockedPopTx = {
        index: nextIndex,
        action: nextAction,
        location: nextLocation,
        callback
      };
      blockers.call(blockedPopTx);
    } else {
      applyTx(nextAction);
    }
  }

  window.addEventListener(PopStateEventType, handlePop);
  let action = Action.Push;
  let [index, location] = getIndexAndLocation();
  let listeners = createEvents();
  let blockers = createEvents();

  if (index == null) {
    index = 0;
    globalHistory.replaceState(_objectSpread(_objectSpread({}, globalHistory.state), {}, {
      idx: index
    }), '');
  }

  function createHref(to) {
    let path = typeof to === 'string' ? to : createPath(to);

    if (hashType != null) {
      let slashChar = path.substr(0, 1);

      if (hashType === 'slash') {
        if (slashChar !== '/') path = '/' + path;
      } else if (hashType === 'noslash') {
        if (slashChar === '/') path = path.substr(1);
      }
    }

    return path;
  }

  function getNextLocation(to, state = null) {
    return readOnly(_objectSpread(_objectSpread(_objectSpread({}, location), typeof to === 'string' ? parsePath(to) : to), {}, {
      state,
      key: createKey()
    }));
  }

  function getHistoryState(nextLocation, index) {
    return {
      usr: nextLocation.state,
      key: nextLocation.key,
      idx: index
    };
  }

  function getHistoryStateAndUrl(nextLocation, index) {
    return [getHistoryState(nextLocation, index), createHref(nextLocation)];
  }

  function allowTx(action, location, callback) {
    return !blockers.length || (blockers.call({
      action,
      location,
      callback
    }), false);
  }

  function applyTx(nextAction) {
    action = nextAction;
    [index, location] = getIndexAndLocation();
    listeners.call({
      action,
      index,
      location
    });
  }

  function pushHistoryState(location, index) {
    let [historyState, url] = getHistoryStateAndUrl(location, index); // TODO: Support forced reloading
    // try...catch because iOS limits us to 100 pushState calls :/

    try {
      globalHistory.pushState(historyState, '', url);
    } catch (error) {
      // They are going to lose state here, but there is no real
      // way to warn them about it since the page will refresh...
      window.location.assign(url);
    }

    return index;
  }

  function replaceHistoryState(location, index) {
    let [historyState, url] = getHistoryStateAndUrl(location, index); // TODO: Support forced reloading

    globalHistory.replaceState(historyState, '', url);
    return index;
  }

  function push(to, state) {
    let nextAction = Action.Push;
    let nextLocation = getNextLocation(to, state);

    const callback = ok => {
      if (!ok) return;

      const _cb = index => {
        pushHistoryState(nextLocation, index + 1);
        applyTx(nextAction);
      };

      if (typeof to !== 'string' && to.delta) {
        blockedPopDc = ({
          index
        }) => _cb(index);

        go(to.delta);
        return;
      }

      _cb(index);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true);
    }
  }

  function replace(to, state) {
    let nextAction = Action.Replace;
    let nextLocation = getNextLocation(to, state);

    const callback = ok => {
      if (!ok) return;

      const _cb = index => {
        replaceHistoryState(nextLocation, index);
        applyTx(nextAction);
      };

      if (typeof to !== 'string' && to.delta) {
        blockedPopDc = ({
          index
        }) => _cb(index);

        go(to.delta);
        return;
      }

      _cb(index);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true);
    }
  }

  function go(delta) {
    globalHistory.go(delta);
  }

  let history = {
    get action() {
      return action;
    },

    get location() {
      return location;
    },

    get index() {
      return index;
    },

    createHref,
    getIndexAndLocation,
    push,
    replace,

    replaceState(state) {
      const historyState = getHistoryState(location, index);
      historyState.usr = state;
      globalHistory.replaceState(historyState, '');
      return state;
    },

    go,

    back() {
      go(-1);
    },

    forward() {
      go(1);
    },

    listen(listener) {
      return listeners.push(listener);
    },

    block(blocker) {
      let unblock = blockers.push(blocker);
      return function () {
        unblock();
      };
    }

  };
  return history;
} // //////////////////////////////////////////////////////////////////////////////
// HASH
// //////////////////////////////////////////////////////////////////////////////

/**
 * Hash history stores the location in window.location.hash. This makes it ideal
 * for situations where you don't want to send the location to the server for
 * some reason, either because you do cannot configure it or the URL space is
 * reserved for something else.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#createhashhistory
 */
export function createHashHistory(options = {}) {
  let {
    window = document.defaultView,
    hashType = 'slash'
  } = options;
  let globalHistory = window.history;

  function getIndexAndLocation() {
    let locationHash = window.location.hash.substr(1);
    if (locationHash && !locationHash.startsWith('/')) locationHash = '/' + locationHash;
    let {
      pathname = '/',
      search = '',
      hash = ''
    } = parsePath(locationHash);
    let state = globalHistory.state || {};
    return [state.idx, readOnly({
      pathname,
      search,
      hash,
      state: state.usr || null,
      key: state.key || 'default'
    })];
  }

  let blockedPopTx = null;
  let blockedPopAp = null;
  let blockedPopDc = null;

  function handlePop(e) {
    let [nextIndex, nextLocation] = getIndexAndLocation();

    if (nextIndex == null) {
      if (index == null || isNaN(index)) index = 0;
      nextIndex = replaceHistoryState(nextLocation, index + 1);
    }

    let delta = index - nextIndex;
    let nextAction = delta > 0 ? Action.Pop : Action.Push;

    if (blockedPopDc) {
      blockedPopDc({
        index: nextIndex,
        location: nextLocation
      });
      blockedPopDc = null;
    } else if (blockedPopAp) {
      if (nextIndex === blockedPopAp.prevIndex) {
        go(blockedPopAp.delta);
        return;
      }

      blockedPopAp = null;
    } else if (blockers.length && delta) {
      const callback = ok => {
        blockedPopTx = null;

        if (ok) {
          applyTx(nextAction);
          return;
        }

        blockedPopAp = {
          action,
          index,
          location,
          prevIndex: nextIndex,
          delta
        };
        go(blockedPopAp.delta);
      };

      blockedPopTx = {
        index: nextIndex,
        action: nextAction,
        location: nextLocation,
        callback
      };
      blockers.call(blockedPopTx);
    } else {
      applyTx(nextAction);
    }
  }

  window.addEventListener(PopStateEventType, handlePop); // popstate does not fire on hashchange in IE 11 and old (trident) Edge
  // https://developer.mozilla.org/de/docs/Web/API/Window/popstate_event

  window.addEventListener(HashChangeEventType, e => {
    if (blockedPopTx || blockedPopAp) return;
    let [, nextLocation] = getIndexAndLocation(); // Ignore extraneous hashchange events.

    if (createPath(nextLocation) !== createPath(location)) {
      handlePop(e);
    }
  });
  let action = Action.Push;
  let [index, location] = getIndexAndLocation();
  let listeners = createEvents();
  let blockers = createEvents();

  if (index == null) {
    index = 0;
    globalHistory.replaceState(_objectSpread(_objectSpread({}, globalHistory.state), {}, {
      idx: index
    }), '');
  }

  function createHref(to) {
    let path = typeof to === 'string' ? to : createPath(to);
    let slashChar = path.substr(0, 1);

    if (hashType === 'slash') {
      if (slashChar !== '/') path = '/' + path;
    } else if (hashType === 'noslash') {
      if (slashChar === '/') path = path.substr(1);
    }

    if (!path.startsWith('#')) path = '#' + path;
    return getBaseHref() + path;
  }

  function getNextLocation(to, state = null) {
    return readOnly(_objectSpread(_objectSpread(_objectSpread({}, location), typeof to === 'string' ? parsePath(to) : to), {}, {
      state,
      key: createKey()
    }));
  }

  function getHistoryState(nextLocation, index) {
    return {
      usr: nextLocation.state,
      key: nextLocation.key,
      idx: index
    };
  }

  function getHistoryStateAndUrl(nextLocation, index) {
    return [getHistoryState(nextLocation, index), createHref(nextLocation)];
  }

  function allowTx(action, location, callback) {
    return !blockers.length || (blockers.call({
      action,
      location,
      callback
    }), false);
  }

  function applyTx(nextAction) {
    action = nextAction;
    let prevIndex = index;
    [index, location] = getIndexAndLocation();

    if (index == null && prevIndex != null) {
      index = nextAction === Action.Push ? prevIndex + 1 : nextAction === Action.Replace ? prevIndex : prevIndex - 1;
    }

    listeners.call({
      action,
      index,
      location
    });
  }

  function pushHistoryState(location, index) {
    let [historyState, url] = getHistoryStateAndUrl(location, index); // TODO: Support forced reloading
    // try...catch because iOS limits us to 100 pushState calls :/

    try {
      globalHistory.pushState(historyState, '', url);
    } catch (error) {
      // They are going to lose state here, but there is no real
      // way to warn them about it since the page will refresh...
      window.location.assign(url);
    }

    return index;
  }

  function replaceHistoryState(location, index) {
    let [historyState, url] = getHistoryStateAndUrl(location, index); // TODO: Support forced reloading

    globalHistory.replaceState(historyState, '', url);
    return index;
  }

  function push(to, state) {
    let nextAction = Action.Push;
    let nextLocation = getNextLocation(to, state);

    const callback = ok => {
      if (!ok) return;

      const _cb = index => {
        pushHistoryState(nextLocation, index + 1);
        applyTx(nextAction);
      };

      if (typeof to !== 'string' && to.delta) {
        blockedPopDc = ({
          index
        }) => _cb(index);

        go(to.delta);
        return;
      }

      _cb(index);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true);
    }
  }

  function replace(to, state) {
    let nextAction = Action.Replace;
    let nextLocation = getNextLocation(to, state);

    const callback = ok => {
      if (!ok) return;

      const _cb = index => {
        replaceHistoryState(nextLocation, index);
        applyTx(nextAction);
      };

      if (typeof to !== 'string' && to.delta) {
        blockedPopDc = ({
          index
        }) => _cb(index);

        go(to.delta);
        return;
      }

      _cb(index);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true);
    }
  }

  function go(delta) {
    globalHistory.go(delta);
  }

  let history = {
    get action() {
      return action;
    },

    get location() {
      return location;
    },

    get index() {
      return index;
    },

    get hashType() {
      return hashType;
    },

    createHref,
    getIndexAndLocation,
    push,
    replace,

    replaceState(state) {
      const historyState = getHistoryState(location, index);
      historyState.usr = state;
      globalHistory.replaceState(historyState, '');
      return state;
    },

    go,

    back() {
      go(-1);
    },

    forward() {
      go(1);
    },

    listen(listener) {
      return listeners.push(listener);
    },

    block(blocker) {
      let unblock = blockers.push(blocker);
      return function () {
        unblock();
      };
    }

  };
  return history;
} // //////////////////////////////////////////////////////////////////////////////
// MEMORY
// //////////////////////////////////////////////////////////////////////////////

/**
 * A user-supplied object that describes a location. Used when providing
 * entries to `createMemoryHistory` via its `initialEntries` option.
 */

/**
 * Memory history stores the current location in memory. It is designed for use
 * in stateful non-browser environments like tests and React Native.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#creatememoryhistory
 */
export function createMemoryHistory(options = {}) {
  let {
    initialEntries = ['/'],
    initialIndex
  } = options;
  let entries = initialEntries.map(entry => {
    let location = readOnly(_objectSpread({
      pathname: '/',
      search: '',
      hash: '',
      state: null,
      key: createKey()
    }, typeof entry === 'string' ? parsePath(entry) : entry));
    return location;
  });
  let index = clamp(initialIndex == null ? entries.length - 1 : initialIndex, 0, entries.length - 1);
  let action = Action.Push;
  let location = entries[index];
  let listeners = createEvents();
  let blockers = createEvents();

  function createHref(to) {
    return typeof to === 'string' ? to : createPath(to);
  }

  function getNextLocation(to, state = null) {
    return readOnly(_objectSpread(_objectSpread(_objectSpread({}, location), typeof to === 'string' ? parsePath(to) : to), {}, {
      state,
      key: createKey()
    }));
  }

  function allowTx(action, location, callback) {
    return !blockers.length || (blockers.call({
      action,
      location,
      callback
    }), false);
  }

  function applyTx(nextAction, nextLocation, nextIndex) {
    action = nextAction;
    location = nextLocation;
    index = nextIndex;
    listeners.call({
      action,
      location,
      index
    });
  }

  function push(to, state) {
    let nextAction = Action.Push;
    let nextLocation = getNextLocation(to, state);

    const callback = ok => {
      if (!ok) return;

      if (typeof to !== 'string' && to.delta) {
        index = clamp(index + to.delta, 0, entries.length - 1);
      }

      index += 1;
      entries.splice(index, entries.length, nextLocation);
      applyTx(nextAction, nextLocation, index);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true);
    }
  }

  function replace(to, state) {
    let nextAction = Action.Replace;
    let nextLocation = getNextLocation(to, state);

    const callback = ok => {
      if (!ok) return;

      if (typeof to !== 'string' && to.delta) {
        index = clamp(index + to.delta, 0, entries.length - 1);
      }

      entries[index] = nextLocation;
      applyTx(nextAction, nextLocation, index);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true);
    }
  }

  function go(delta) {
    let nextIndex = clamp(index + delta, 0, entries.length - 1);
    let nextAction = Action.Pop;
    let nextLocation = entries[nextIndex];

    const callback = ok => {
      if (!ok) return;
      index = nextIndex;
      applyTx(nextAction, nextLocation, index);
    };

    if (allowTx(nextAction, nextLocation, callback)) {
      callback(true);
    }
  }

  let history = {
    get action() {
      return action;
    },

    get location() {
      return location;
    },

    get index() {
      return index;
    },

    createHref,

    getIndexAndLocation() {
      return [index, location];
    },

    push,
    replace,

    replaceState(state) {
      return location.state = state;
    },

    go,

    back() {
      go(-1);
    },

    forward() {
      go(1);
    },

    listen(listener) {
      return listeners.push(listener);
    },

    block(blocker) {
      return blockers.push(blocker);
    }

  };
  return history;
} // //////////////////////////////////////////////////////////////////////////////
// UTILS
// //////////////////////////////////////////////////////////////////////////////

function clamp(n, lowerBound, upperBound) {
  return Math.min(Math.max(n, lowerBound), upperBound);
} // function promptBeforeUnload(event: BeforeUnloadEvent) {
//   // Cancel the event.
//   event.preventDefault();
//   // Chrome (and legacy IE) requires returnValue to be set.
//   event.returnValue = '';
// }


function createEvents() {
  let handlers = [];
  return {
    get length() {
      return handlers.length;
    },

    push(fn) {
      handlers.push(fn);
      return function () {
        handlers = handlers.filter(handler => handler !== fn);
      };
    },

    call(arg) {
      handlers.forEach(fn => fn && fn(arg));
    }

  };
}

function createKey() {
  return Math.random().toString(36).substr(2, 8);
}

export function getBaseHref() {
  let base = document.querySelector('base');
  let href = '';

  if (base && base.getAttribute('href')) {
    let url = window.location.href;
    let hashIndex = url.indexOf('#');
    href = hashIndex === -1 ? url : url.slice(0, hashIndex);
  }

  return href;
}
/**
 * Creates a string URL path from the given pathname, search, and hash components.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#createpath
 */

export function createPath({
  pathname = '/',
  search = '',
  hash = ''
}) {
  return pathname + search + hash;
}
/**
 * Parses a string URL path into its separate pathname, search, and hash components.
 *
 * @see https://github.com/ReactTraining/history/tree/master/docs/api-reference.md#parsepath
 */

export function parsePath(path) {
  let partialPath = {};

  if (path) {
    let hashIndex = path.indexOf('#');

    if (hashIndex >= 0) {
      partialPath.hash = path.substr(hashIndex);
      path = path.substr(0, hashIndex);
      if (path && !path.startsWith('/')) path = '/' + path;
    }

    let searchIndex = path.indexOf('?');

    if (searchIndex >= 0) {
      partialPath.search = path.substr(searchIndex);
      path = path.substr(0, searchIndex);
      if (path && !path.startsWith('/')) path = '/' + path;
    }

    if (path) {
      partialPath.pathname = path;
    }
  }

  return partialPath;
}