function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import { RouterContext, RouterViewContext } from './context';

function withRouter(comp, {
  withRoute = false
} = {}) {
  return React.forwardRef((props, ref) => React.createElement(RouterContext.Consumer, {}, router => {
    const newProps = _objectSpread(_objectSpread({}, props), {}, {
      ref,
      router: router || null
    });

    if (withRoute) newProps.route = router ? router.currentRoute : null;
    return React.createElement(comp, newProps, props.children);
  }));
}

function withRoute(comp, {
  withRouter = false
} = {}) {
  return React.forwardRef((props, ref) => React.createElement(RouterContext.Consumer, {}, router => {
    const newProps = _objectSpread(_objectSpread({}, props), {}, {
      ref,
      route: router ? router.currentRoute : null
    });

    if (withRouter) newProps.router = router || null;
    return React.createElement(comp, newProps, props.children);
  }));
}

function withRouterView(comp) {
  return React.forwardRef((props, ref) => React.createElement(RouterViewContext.Consumer, {}, routerView => {
    const newProps = _objectSpread(_objectSpread({}, props), {}, {
      ref,
      routerView: routerView || null
    });

    return React.createElement(comp, newProps, props.children);
  }));
}

function withMatchedRouteIndex(comp, {
  withMatchedRoute = false
} = {}) {
  return React.forwardRef((props, ref) => React.createElement(RouterViewContext.Consumer, {}, routerView => {
    const matchedRouteIndex = routerView ? routerView.state._routerDepth : -1;

    const newProps = _objectSpread(_objectSpread({}, props), {}, {
      ref,
      matchedRouteIndex
    });

    if (withMatchedRoute) newProps.matchedRoute = routerView ? routerView.state.currentRoute : null;
    return React.createElement(comp, newProps, props.children);
  }));
}

function withMatchedRoute(comp, {
  withMatchedRouteIndex = false
} = {}) {
  return React.forwardRef((props, ref) => React.createElement(RouterViewContext.Consumer, {}, routerView => {
    const newProps = _objectSpread(_objectSpread({}, props), {}, {
      ref,
      matchedRoute: routerView ? routerView.state.currentRoute : null
    });

    if (withMatchedRouteIndex) newProps.matchedRouteIndex = routerView ? routerView.state._routerDepth : -1;
    return React.createElement(comp, newProps, props.children);
  }));
}

export { withRouter, withRoute, withRouterView, withMatchedRouteIndex, withMatchedRoute };