function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { useContext, useState, useEffect, useCallback, useImperativeHandle } from 'react';
import { RouterContext, RouterViewContext } from '../context';
import { innumerable, isFunction, isPlainObject } from '../util';

function useRouter(defaultRouter) {
  // eslint-disable-next-line react-hooks/rules-of-hooks
  return defaultRouter || useContext(RouterContext);
}

function useRoute(defaultRouter) {
  const router = useRouter(defaultRouter);
  return router ? router.currentRoute || router.initialRoute : null;
}

function useRouteMeta(metaKey, defaultRouter) {
  const router = useRouter(defaultRouter);
  const route = useMatchedRoute(router);
  const meta = route && route.meta;
  const keyIsArray = Array.isArray(metaKey);
  if (keyIsArray && !metaKey.length) throw new Error('metaKey is Empty!');
  const [value, _setValue] = useState(() => keyIsArray ? metaKey.reduce((p, key) => {
    if (!meta) return p;
    p[key] = meta[key];
    return p;
  }, {}) : {
    [metaKey]: meta && meta[metaKey]
  });
  const setValue = useCallback((newValue, setAll = false) => {
    if (!meta) return;

    if (!keyIsArray) {
      newValue = {
        [metaKey]: newValue
      };
    } else if (!setAll) {
      newValue = Object.keys(newValue).reduce((p, key) => {
        if (metaKey.includes(key)) p[key] = newValue[key];
        return p;
      }, {});
    }

    let changed = route && router && router.updateRouteMeta(route.config, newValue);
    if (!changed) return;

    _setValue(_objectSpread(_objectSpread({}, value), newValue));
  }, [meta, keyIsArray, route, router, value, metaKey]);
  return keyIsArray ? [value, setValue] : [value[metaKey], setValue];
}

function useRouterView() {
  return useContext(RouterViewContext);
}

function useMatchedRouteIndex() {
  const view = useRouterView();
  return view ? view.state._routerDepth : 0;
}

function useMatchedRoute(defaultRouter) {
  const route = useRoute(defaultRouter);
  const routeIndex = useMatchedRouteIndex();
  return route && route.matched[routeIndex] || null;
}

function useRouteState(defaultRouter) {
  const router = useRouter(defaultRouter);
  const route = useMatchedRoute(router || defaultRouter);
  const routeUrl = route && route.url;
  const setRouteState = useCallback(newState => router && router.replaceState(newState, route || undefined), // eslint-disable-next-line react-hooks/exhaustive-deps
  [router, routeUrl]);
  return [route ? route.state : {}, setRouteState];
}

function useRouteChanged(router, onChange) {
  return useEffect(() => {
    const unplugin = onChange && router.plugin(onChange);
    return () => unplugin && unplugin();
  }, [router, onChange]);
}

function useRouteMetaChanged(router, onChange, deps = []) {
  return useEffect(() => {
    const unplugin = onChange && router.plugin({
      onRouteMetaChange(newVal, oldVal, route, router) {
        if (deps.length && !deps.some(v => {
          if (isPlainObject(v)) return v === route.meta;
          return v in oldVal;
        })) return;
        return onChange(newVal, oldVal, route, router);
      }

    });
    return () => unplugin && unplugin(); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [router, onChange, ...deps]);
}

function createRouteGuardsRef(ref) {
  if (ref && !ref.__routeGuardInfoHooks) innumerable(ref, '__routeGuardInfoHooks', true);
  return ref;
}

function useRouteGuardsRef(ref, guards, deps = []) {
  if (!isFunction(guards)) deps.concat(guards);
  return useImperativeHandle(ref, (...args) => {
    let ret;
    if (isFunction(guards)) ret = guards(...args);else ret = guards;
    return createRouteGuardsRef(ret);
  }, // eslint-disable-next-line react-hooks/exhaustive-deps
  deps);
}

export { useRouter, useRouteChanged, useRouteMetaChanged, useRoute, useRouteMeta, useRouteState, useRouterView, useMatchedRouteIndex, useMatchedRoute, useRouteGuardsRef, createRouteGuardsRef };