function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import config from './config';
import { RouteLazy } from './route-lazy';
import { REACT_FORWARD_REF_TYPE, getGuardsComponent } from './route-guard';
import matchPath, { computeRootMatch } from './match-path';
import { RouterViewWrapper } from './router-view';

function nextTick(cb, ctx) {
  if (!cb) return;
  return new Promise(function (resolve) {
    setTimeout(() => resolve(ctx ? cb.call(ctx) : cb()), 0);
  });
}

function readonly(obj, key, value, options = {
  configurable: true,
  enumerable: true
}) {
  Object.defineProperty(obj, key, _objectSpread({
    get() {
      return value;
    }

  }, options));
  return obj;
}

function innumerable(obj, key, value, options = {
  configurable: true
}) {
  Object.defineProperty(obj, key, _objectSpread({
    value
  }, options));
  return obj;
}

function normalizePath(path) {
  const paths = path.split('/');
  if (paths.length > 2 && !paths[paths.length - 1]) paths.splice(paths.length - 1, 1);

  for (let i = paths.length - 1; i > -1; i--) {
    if (paths[i + 1] === '..') paths.splice(i, 2);else if (paths[i] === '.') paths.splice(i, 1);
  }

  return paths.join('/').replace(/\/{2,}/, '/');
}

function normalizeRoute(route, parent, options = {}) {
  let path = normalizePath(parent ? `${parent.path}${route.path === '/' ? '' : '/'}${route.path.replace(/^(\/)/, '')}` : route.path);

  let r = _objectSpread(_objectSpread({}, route), {}, {
    subpath: route.path,
    path: parent ? path : path[0] === '/' ? path : `/${path}`,
    depth: parent ? parent.depth + 1 : 0
  });

  if (parent) innumerable(r, 'parent', parent);
  if (!r.children) r.children = [];

  if (isFunction(r.children)) {
    if (!r.children._normalized) {
      const fn = r.children;
      const ctx = {};

      r.children = function () {
        return fn.apply(ctx, arguments);
      };

      innumerable(r.children, '_ctx', ctx);
      innumerable(r.children, '_normalized', true);
    }
  } else {
    innumerable(r, 'children', normalizeRoutes(r.children, r, options));
  }

  r.exact = r.exact !== undefined ? r.exact : Boolean(r.redirect || r.index);
  if (!r.components) r.components = {};

  if (r.component) {
    r.components.default = r.component;
    delete r.component;
  }

  Object.keys(r.components).forEach(key => {
    let comp = r.components[key];

    if (comp instanceof RouteLazy) {
      comp.updaters.push(c => {
        if (c && c.__children) {
          let children = c.__children || [];
          if (isFunction(children)) children = children(r) || [];
          innumerable(r, 'children', normalizeRoutes(children, r));
          r.exact = !r.children.length;
        }

        return r.components[key] = c;
      });
    }
  });
  readonly(r, 'meta', r.meta || {});
  if (r.props) innumerable(r, 'props', normalizeProps(r.props));
  if (r.paramsProps) innumerable(r, 'paramsProps', normalizeProps(r.paramsProps));
  if (r.queryProps) innumerable(r, 'queryProps', normalizeProps(r.queryProps));
  innumerable(r, '_pending', {
    completeCallbacks: {}
  });
  return r;
}

function walkRoutes(routes, walkFn) {
  return routes.some((route, routeIndex) => {
    if (walkFn(route, routeIndex, routes)) return true;
    if (!route.children || !Array.isArray(route.children)) return;
    return walkRoutes(route.children, walkFn);
  });
}

function normalizeRoutes(routes, parent, options = {}) {
  if (!routes) routes = [];
  if (!options.force && routes._normalized) return routes;
  routes = routes.map(route => route && normalizeRoute(route, parent, options)).filter(Boolean);
  Object.defineProperty(routes, '_normalized', {
    value: true
  });
  return routes;
}

function normalizeRoutePath(path, route, append, basename = '') {
  if (isAbsoluteUrl(path)) return path;
  if (isRoute(route)) route = route.matched[route.matched.length - 1];
  if (!path || ['/', '#'].includes(path[0]) || !route) return normalizePath(basename + (path || ''));
  if (isMatchedRoute(route)) route = route.config;
  let parent = append || /^\.\//.test(path) ? route : route.parent || {
    path: ''
  };

  while (parent && path[0] !== '/') {
    path = `${parent.path}/${path}`;
    parent = route.parent;
  }

  if (basename && path[0] === '/') path = basename + path;
  return normalizePath(path);
}

function resloveIndex(_index, routes) {
  let index = isFunction(_index) ? _index(routes) : _index;
  if (!index) return null;
  let r = routes.find(r => {
    let path1 = r.subpath[0] === '/' ? r.subpath : `/${r.subpath}`;
    let path2 = index[0] === '/' ? index : `/${index}`;
    return path1 === path2;
  }) || null;

  if (r && r.index) {
    if (r.index === _index) return null;
    return resloveIndex(r.index, routes);
  }

  return r;
}

function matchRoutes(routes, to, parent, branch = [], level = 0) {
  to = normalizeLocation(to);
  if (!to || to.path === '') return [];

  if (isFunction(routes)) {
    let fn = routes;
    routes = normalizeRoutes(fn({
      to,
      parent,
      branch,
      prevChildren: fn._ctx && fn._ctx.prevChildren
    }), parent);
    if (fn._ctx) fn._ctx.prevChildren = routes;
  }

  routes.some(route => {
    let match = route.path ? matchPath(to.path, route) : branch.length ? branch[branch.length - 1].match // use parent match
    : computeRootMatch(to.path); // use default "root" match

    if (match && route.index) {
      route = resloveIndex(route.index, routes);
      if (!route) return;
      to.pathname = to.path = route.path;
      match = matchPath(route.path, route);
    }

    if (!match) return;
    branch.push({
      route,
      match
    });
    if (route.children && (route.children.length || isFunction(route.children))) matchRoutes(route.children, to, route, branch, level + 1);
    return true;
  });

  if (!level && branch.length) {
    const lastRouteBranch = branch[branch.length - 1];
    branch.unmatchedPath = to.pathname.substr(lastRouteBranch.match.path.length, to.pathname.length);
  }

  return branch;
}

function normalizeLocation(to, route, {
  append = false,
  basename = '',
  mode = '',
  resolvePathCb
} = {}) {
  if (!to || isPlainObject(to) && !to.path && !to.pathname) return null;
  if (to._routeNormalized) return to;

  if (typeof to === 'string') {
    const searchs = to.match(/\?[^#]+/g) || [];
    const pathname = searchs.reduce((p, v) => p.replace(v, ''), to);
    const search = searchs.reduce((p, v, i) => {
      if (!i) return v;
      const s = v.substr(1);
      return p + (s ? `&${s}` : '');
    }, '');
    to = {
      pathname,
      path: pathname,
      search,
      fullPath: to
    };
  }

  if (to.query) Object.keys(to.query).forEach(key => to.query[key] === undefined && delete to.query[key]);else if (to.search) to.query = config.parseQuery(to.search);
  let isAbsolute = isAbsoluteUrl(to.pathname);

  if (isAbsolute && mode !== 'browser') {
    let hash = getCurrentPageHash(to.pathname);

    if (hash) {
      to.pathname = hash;
      isAbsolute = false;
    }
  }

  if (to.basename == null && basename) to.basename = to.absolute ? '' : basename;

  if (!isAbsolute) {
    let path = to.pathname || to.path;
    if (resolvePathCb) path = resolvePathCb(path, to);
    path = normalizeRoutePath(path, route, to.append || append, to.absolute ? '' : basename) || '/';
    to.pathname = to.path = path;
  }

  Object.defineProperty(to, 'search', {
    enumerable: true,
    configurable: true,

    get() {
      return config.stringifyQuery(this.query);
    }

  });
  Object.defineProperty(to, 'fullPath', {
    enumerable: true,
    configurable: true,

    get() {
      return `${this.path}${this.search || ''}` || '/';
    }

  });
  if (!to.query) to.query = {};
  innumerable(to, '_normalized', true);
  return to;
}

const _toString = Object.prototype.toString;

function isPlainObject(obj) {
  return _toString.call(obj) === '[object Object]';
}

function isFunction(value) {
  return typeof value === 'function';
}

function isNull(value) {
  return value === null || value === undefined;
}

function isMatchedRoute(value) {
  return Boolean(value && value.config);
}

function isLocation(v) {
  return isPlainObject(v) && (v.path || v.pathname);
}

function isHistoryLocation(v) {
  return isLocation(v) && Boolean(v._routeNormalized);
}

function normalizeProps(props) {
  let res = {};

  if (Array.isArray(props)) {
    props.forEach(key => res[key] = {
      type: null
    });
  } else if (isPlainObject(props)) {
    Object.keys(props).forEach(key => {
      let val = props[key];
      res[key] = isPlainObject(val) ? val.type !== undefined ? val : normalizeProps(val) : {
        type: val
      };
    });
  } else return props;

  return res;
}

function once(fn, ctx) {
  let ret;
  return function _once(...args) {
    if (!fn) return ret;
    const _fn = fn;
    fn = null;
    ret = _fn.call(ctx || this, ...args);
    return ret;
  };
}

function isAcceptRef(v) {
  if (!v) return false;
  if (v.$$typeof === REACT_FORWARD_REF_TYPE && v.__componentClass) return true;
  if (v.__component) v = getGuardsComponent(v);
  let ret = false;

  if (v.prototype) {
    if (v.prototype instanceof React.Component || v.prototype.componentDidMount !== undefined) ret = true;
  } else if (v.$$typeof === REACT_FORWARD_REF_TYPE && !v.__guards) ret = true;

  return ret;
}

function mergeFns(...fns) {
  return function (...args) {
    let ret;
    fns.forEach(fn => {
      ret = fn && fn.call(this, ...args);
    });
    return ret;
  };
}

function resolveRedirect(to, route, from) {
  if (isFunction(to)) to = to.call(route.config, from);
  if (!to) return '';
  let ret = normalizeLocation(to, route);
  if (!ret) return '';
  from && Object.assign(ret.query, from.query);
  ret.isRedirect = true;
  return ret;
}

function warn(...args) {
  console.warn(...args);
}

async function afterInterceptors(interceptors, to, from) {
  for (let i = 0; i < interceptors.length; i++) {
    let interceptor = interceptors[i];

    while (interceptor && interceptor.lazy) {
      interceptor = await interceptor(interceptors, i);
    }

    if (!interceptor) return;
    interceptor && (await interceptor.call(this, to, from, interceptor.route));
  }
}

function renderRoute(route, routes, props, children, options = {}) {
  if (props === undefined) props = {};
  if (!route) return null;
  if (React.isValidElement(route)) return route;
  if (route.config) route = route.config;

  function configProps(_props, configs, obj, name) {
    if (!obj) return;
    if (name && configs[name] !== undefined) configs = configs[name];
    if (configs === true) Object.assign(_props, obj);else if (isPlainObject(configs)) {
      Object.keys(configs).forEach(key => {
        const prop = configs[key];
        const type = prop.type;
        let val = obj[key];

        if (val === undefined) {
          if (prop.default) {
            if (isFunction(prop.default) && (type === Object || type === Array)) {
              _props[key] = prop.default();
            } else _props[key] = prop.default;
          } else return;
        }

        if (type !== null) _props[key] = type(val);else _props[key] = val;
      });
    }
  }

  function createComp(route, props, children, options) {
    let component = route.components && route.components[options.name || 'default'];

    if (!component) {
      if (route.children && route.children.length && options.router) {
        component = RouterViewWrapper;
      } else return null;
    }

    if (isMatchedRoute(route)) route = route.config;
    const _props = {
      key: route.path
    };

    if (route.defaultProps) {
      Object.assign(_props, isFunction(route.defaultProps) ? route.defaultProps(props) : route.defaultProps);
    }

    if (route.props) configProps(_props, route.props, options.params, options.name);
    if (route.paramsProps) configProps(_props, route.paramsProps, options.params, options.name);
    if (route.queryProps) configProps(_props, route.queryProps, options.query, options.name);
    let ref = null;

    if (component) {
      if (isAcceptRef(component)) ref = options.ref;else if (route.enableRef) {
        if (!isFunction(route.enableRef) || route.enableRef(component)) ref = options.ref;
      }
    }

    const _pending = route._pending;
    const completeCallback = _pending && _pending.completeCallbacks[options.name || 'default'];
    let refHandler = once((el, componentClass) => {
      if (el || !ref) {
        // if (isFunction(componentClass)) componentClass = componentClass(el, route);
        if (componentClass && el && (el._reactInternalFiber || el._reactInternals)) {
          let refComp = null;
          let comp = el._reactInternalFiber || el._reactInternals;

          while (comp && !refComp) {
            if (comp.type === componentClass) {
              refComp = comp;
              break;
            }

            comp = comp.child;
          }

          if (refComp && refComp.stateNode instanceof componentClass) el = refComp.stateNode;else warn('componentClass', componentClass, 'not found in route component: ', el);
        }

        completeCallback && completeCallback(el);
      }
    });
    _pending && (_pending.completeCallbacks[options.name || 'default'] = null);
    if (ref) ref = mergeFns(ref, el => el && refHandler && refHandler(el, component.__componentClass));
    if (component.__component) component = getGuardsComponent(component);
    let ret;

    if (component instanceof RouteLazy) {
      ret = null;
      warn(`route [${route.path}] component should not be RouteLazy instance!`);
    } else {
      ret = React.createElement(component, Object.assign(_props, props, config.inheritProps ? {
        route
      } : null, {
        ref
      }), ...(Array.isArray(children) ? children : [children]));
    }

    if (!ref) nextTick(refHandler);
    return ret;
  }

  let renderRoute = route;
  if (route && route.redirect) return null;
  if (route && route.index) renderRoute = resloveIndex(route.index, routes);
  if (!renderRoute) return null;
  let result = createComp(renderRoute, props, children, options);
  if (route && options.container) result = options.container(result, route, props);
  return result;
}

function flatten(array) {
  let flattend = [];

  (function flat(array) {
    array.forEach(function (el) {
      if (Array.isArray(el)) flat(el);else flattend.push(el);
    });
  })(array);

  return flattend;
}

function camelize(str) {
  let ret = str.replace(/[-](\w)/g, function (_, c) {
    return c ? c.toUpperCase() : '';
  });
  if (/^[A-Z]/.test(ret)) ret = ret.charAt(0).toLowerCase() + ret.substr(1);
  return ret;
}

function isPropChanged(prev, next, onChanged) {
  if ((!prev || !next) && prev !== next) return true;
  return Object.keys(next).some(key => {
    let newVal = next[key];
    let oldVal = prev[key];
    let changed = newVal !== oldVal;
    if (changed && onChanged) changed = onChanged(key, newVal, oldVal);
    return changed;
  });
}

function isRouteChanged(prev, next) {
  if (prev && next) return prev.path !== next.path || prev.subpath !== next.subpath;
  if ((!prev || !next) && prev !== next) return true;
  return false;
}

function isRoutesChanged(prevs, nexts) {
  if (!prevs || !nexts) return true;
  if (prevs.length !== nexts.length) return true;
  let changed = false;
  prevs.some((prev, i) => {
    changed = isRouteChanged(prev, nexts[i]);
    return changed;
  });
  return changed;
}

function getHostRouterView(ctx, continueCb) {
  let parent = (ctx._reactInternalFiber || ctx._reactInternals).return;

  while (parent) {
    if (continueCb && continueCb(parent) === false) return null;
    const memoizedState = parent.memoizedState; // const memoizedProps = parent.memoizedProps;

    if (memoizedState && memoizedState._routerView) return parent.stateNode;
    parent = parent.return;
  }

  return null;
}

function getParentRoute(ctx) {
  const view = getHostRouterView(ctx);
  return view && view.state.currentRoute || null;
}

function isAbsoluteUrl(to) {
  return typeof to === 'string' && /^(https?:)?\/\/.+/.test(to);
}

function getCurrentPageHash(to) {
  if (!to || !window || !window.location) return '';
  let [, host = '', hash = ''] = to.match(/(.+)#(.+)$/) || [];
  return window.location.href.startsWith(host) ? hash : '';
}

function getSessionStorage(key, json = false) {
  if (!window || !window.sessionStorage) return null;
  let v = window.sessionStorage[key];
  if (v === undefined) return json ? null : '';
  return json ? JSON.parse(v) : v;
}

function setSessionStorage(key, value) {
  if (!window || !window.sessionStorage) return;
  let isNull = value === undefined || value === null;
  let v = typeof value === 'string' ? value : JSON.stringify(value);
  if (!v || isNull) window.sessionStorage.removeItem(key);else window.sessionStorage[key] = v;
}

function isRoute(route) {
  return Boolean(route && route.isViewRoute);
}

function isReactViewRouter(v) {
  return v && v.isReactViewRouterInstance;
}

function isHistory(v) {
  return v && v.isHistoryInstance;
}

function isRouteGuardInfoHooks(v) {
  return v && v.__routeGuardInfoHooks;
}

function isReadonly(obj, key) {
  let d = obj && Object.getOwnPropertyDescriptor(obj, key);
  return Boolean(!obj || d && !d.writable);
}

export { camelize, flatten, warn, once, mergeFns, isAcceptRef, nextTick, isNull, isPlainObject, isFunction, isMatchedRoute, isLocation, isHistoryLocation, isPropChanged, isRouteChanged, isRoutesChanged, isAbsoluteUrl, isRoute, isReactViewRouter, isRouteGuardInfoHooks, isHistory, isReadonly, resolveRedirect, normalizePath, normalizeRoute, normalizeRoutes, walkRoutes, normalizeRoutePath, normalizeLocation, normalizeProps, matchPath, matchRoutes, renderRoute, innumerable, readonly, afterInterceptors, getParentRoute, getHostRouterView, getCurrentPageHash, getSessionStorage, setSessionStorage };