"use strict";

require("core-js/modules/es6.symbol.js");

require("core-js/modules/es6.string.iterator.js");

require("core-js/modules/es6.object.to-string.js");

require("core-js/modules/es6.array.iterator.js");

require("core-js/modules/web.dom.iterable.js");

require("core-js/modules/es6.array.slice.js");

require("core-js/modules/es6.array.from.js");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.readRouteMeta = readRouteMeta;
exports.walkRouteTitle = walkRouteTitle;
exports.default = void 0;

require("core-js/modules/es6.array.filter.js");

require("core-js/modules/es6.array.map.js");

var _react = require("react");

var _base = require("./base");

var _util = require("../util");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function readRouteMeta(route) {
  var key = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var props = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var value = key && route.meta[key];

  if ((0, _util.isFunction)(value)) {
    value = value(route, route.parent ? route.parent.children : props.router && props.router.routes, props);
  }

  return value;
}

function walkRouteTitle(router) {
  var routes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var refresh = arguments.length > 2 ? arguments[2] : undefined;
  var filter = arguments.length > 3 ? arguments[3] : undefined;
  var maxLevel = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 99;
  var level = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : 1;
  return routes.filter(function (r) {
    return r.meta.title;
  }).map(function (r) {
    var visible = readRouteMeta(r, 'visible', {
      router: router,
      level: level,
      maxLevel: maxLevel,
      refresh: refresh
    });
    if (visible === false) return;
    var title = readRouteMeta(r, 'title', {
      router: router,
      level: level,
      maxLevel: maxLevel,
      refresh: refresh
    });
    if (!title) return;
    if (filter && filter(r, routes, {
      title: title,
      visible: visible,
      router: router,
      level: level,
      maxLevel: maxLevel,
      refresh: refresh
    }) === false) return;
    var ret = {
      title: title,
      path: r.path,
      meta: r.meta
    };

    if (level < maxLevel && Array.isArray(r.children) && r.children.length) {
      var children = walkRouteTitle(router, r.children, refresh, filter, maxLevel, level + 1);
      if (children.length) ret.children = children;
    }

    return ret;
  }).filter(Boolean);
}

function getMatched(router, currentRoute) {
  var matched = currentRoute.matched;

  if (matched) {
    var isCommonPage = matched.some(function (r) {
      return r.meta.commonPage;
    });

    if (isCommonPage && currentRoute.query.redirect) {
      matched = router.getMatched(currentRoute.query.redirect);
    }
  }

  return matched;
}

function getMatchedDepth(matchedRoute) {
  return matchedRoute ? matchedRoute.depth + 1 : 0;
}

function getMatchedPathList(matched, depth, maxLevel) {
  var ret = [];

  while (depth < maxLevel && matched[depth]) {
    var path = matched[depth++].path;
    ret.push(path);
  }

  return ret;
}

var dirtyTimerId;

function useRouteTitle() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var defaultRouter = arguments.length > 1 ? arguments[1] : undefined;
  var deps = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  var _props$maxLevel = props.maxLevel,
      maxLevel = _props$maxLevel === void 0 ? 99 : _props$maxLevel,
      filter = props.filter,
      _props$filterMetas = props.filterMetas,
      filterMetas = _props$filterMetas === void 0 ? [] : _props$filterMetas,
      manual = props.manual;
  var router = (0, _base.useRouter)(defaultRouter); // eslint-disable-next-line react-hooks/rules-of-hooks

  var matchedRoute = defaultRouter ? null : (0, _base.useMatchedRoute)();

  var _useState = (0, _react.useState)(false),
      _useState2 = _slicedToArray(_useState, 2),
      dirty = _useState2[0],
      setDirty = _useState2[1];

  var refreshTitles = (0, _react.useCallback)(function () {
    return setDirty(true);
  }, [setDirty]);
  var refreshTabs = (0, _react.useCallback)(function () {
    var routes = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    return walkRouteTitle(router, routes, refreshTitles, filter, maxLevel);
  }, // eslint-disable-next-line react-hooks/exhaustive-deps
  [router, filter, maxLevel, refreshTitles].concat(_toConsumableArray(deps)));

  var _useState3 = (0, _react.useState)(function () {
    return manual ? [] : refreshTabs(matchedRoute ? matchedRoute.config.children : router.routes);
  }),
      _useState4 = _slicedToArray(_useState3, 2),
      titles = _useState4[0],
      setTitles = _useState4[1];

  var _useState5 = (0, _react.useState)(function () {
    var currentRoute = router.currentRoute || router.initialRoute;
    var depth = getMatchedDepth(matchedRoute);
    return getMatchedPathList(getMatched(router, currentRoute), depth, maxLevel + depth);
  }),
      _useState6 = _slicedToArray(_useState5, 2),
      currentPaths = _useState6[0],
      setCurrentPaths = _useState6[1];

  (0, _base.useRouteChanged)(router, (0, _react.useCallback)(function (route) {
    var depth = getMatchedDepth(matchedRoute);
    var paths = getMatchedPathList(getMatched(router, route), depth, maxLevel + depth);

    if (paths.length !== currentPaths.length || currentPaths.some(function (path, i) {
      return path !== paths[i];
    })) {
      setCurrentPaths(paths);
    }
  }, [router, matchedRoute, maxLevel, currentPaths, setCurrentPaths]));
  var routeMetaChangedCallback = (0, _react.useCallback)(function () {
    return setTitles(refreshTabs(matchedRoute ? matchedRoute.config.children : router.routes));
  }, [matchedRoute, setTitles, refreshTabs, router.routes]);
  (0, _base.useRouteMetaChanged)(router, routeMetaChangedCallback, [].concat(_toConsumableArray(filterMetas), ['title', 'visible']));
  (0, _react.useMemo)(function () {
    if (!dirty) return;
    setDirty(false);
    if (dirtyTimerId) return;
    dirtyTimerId = setTimeout(function () {
      dirtyTimerId = null;
      router.isRunning && routeMetaChangedCallback();
    }, 0);
  }, [router, dirty, routeMetaChangedCallback]);
  return {
    titles: titles,
    setTitles: setTitles,
    refreshTitles: refreshTitles,
    currentPaths: currentPaths,
    setCurrentPaths: setCurrentPaths
  };
}

var _default = useRouteTitle;
exports.default = _default;
//# sourceMappingURL=use-route-title.js.map